@@ -1,5 +1,5 @@
 
-/*  A Bison parser, made from /home/gray/radius/lib/./users.y
+/*  A Bison parser, made from /home/gray/gnu/radius/lib/./users.y
     by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -18,7 +18,9 @@
 #line 1 "users.y"
 
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Sergey Poznyakoff
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
   
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -54,17 +56,15 @@
 #include <errno.h>
 #include <sys/wait.h>
 
-#include <sysdep.h>
-#include <radius.h>
-#include <parser.h>
+#include <common.h>
 
 #define YYMAXDEPTH 10
 
-int old_lineno;
+static grad_locus_t start_loc;
 
 static void *closure;
-static int (*add_entry)(void*, char *, int, char *, VALUE_PAIR *, VALUE_PAIR *);
-static VALUE_PAIR *install_pair0(char *name, int op, char *valstr);
+static register_rule_fp add_entry;
+static grad_avp_t *grad_create_pair0(char *name, int op, char *valstr);
 
 int uyyerror(char *s);
 extern int uyylex();
@@ -73,12 +73,12 @@ extern int uyylex();
 #line 67 "users.y"
 typedef union {
         char *string;
-        MATCHING_RULE *rule;
+        grad_matching_rule_t *rule;
         struct {
-                VALUE_PAIR *lhs, *rhs;
+                grad_avp_t *lhs, *rhs;
         } descr;
-        VALUE_PAIR *pair;
-        int op;
+        grad_avp_t *pair;
+        enum grad_operator op;
 } YYSTYPE;
 #ifndef YYDEBUG
 #define YYDEBUG 1
@@ -150,9 +150,9 @@ static const short uyyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short uyyrline[] = { 0,
-    79,    80,    83,    86,    87,    93,    99,   106,   112,   119,
-   123,   126,   127,   138,   142,   148,   152,   156,   160,   164,
-   168,   174,   175
+    79,    80,    83,    86,    87,    93,    97,   109,   115,   122,
+   126,   129,   130,   141,   145,   151,   155,   159,   163,   167,
+   171,   177,   178
 };
 #endif
 
@@ -763,100 +763,103 @@ case 3:
 case 5:
 #line 88 "users.y"
 {
-                   users_sync(); uyyerrok; uyyclearin;
+                   grad_parser_lex_sync(); uyyerrok; uyyclearin;
            ;
     break;}
 case 6:
 #line 94 "users.y"
 {
-                   add_entry(closure,
-                             source_filename,
-                             old_lineno, uyyvsp[-1].string, uyyvsp[0].descr.lhs, uyyvsp[0].descr.rhs);
+                   add_entry(closure, &start_loc, uyyvsp[-1].string, uyyvsp[0].descr.lhs, uyyvsp[0].descr.rhs);
            ;
     break;}
 case 7:
-#line 100 "users.y"
+#line 98 "users.y"
 {
-                   radlog(L_ERR, _("discarding user `%s'"), uyyvsp[-1].string);
-                   users_sync(); uyyerrok; uyyclearin;
+                   grad_log(L_ERR, _("discarding user `%s'"), uyyvsp[-1].string);
+                   if (grad_parser_lex_sync() <= 0)
+			   uyychar = 0; /* force end-of-file */
+		   else {
+			   uyyerrok;
+			   uyyclearin;
+		   }
            ;
     break;}
 case 8:
-#line 107 "users.y"
+#line 110 "users.y"
 {
-                   old_lineno = source_line_num;
+		   start_loc = grad_parser_source_locus;
            ;
     break;}
 case 9:
-#line 113 "users.y"
+#line 116 "users.y"
 {
                    uyyval.descr.lhs = uyyvsp[-1].pair;
                    uyyval.descr.rhs = uyyvsp[0].pair;
            ;
     break;}
 case 10:
-#line 120 "users.y"
+#line 123 "users.y"
 {
                    uyyval.pair = NULL;
            ;
     break;}
 case 13:
-#line 128 "users.y"
+#line 131 "users.y"
 {
                    if (uyyvsp[-2].pair) {
                            if (uyyvsp[0].pair) 
-                                   avl_add_list(&uyyvsp[-2].pair, uyyvsp[0].pair);
+                                   grad_avl_add_list(&uyyvsp[-2].pair, uyyvsp[0].pair);
                            uyyval.pair = uyyvsp[-2].pair;
                    } else
                            uyyval.pair = uyyvsp[0].pair;
            ;
     break;}
 case 14:
-#line 139 "users.y"
+#line 142 "users.y"
 {
-                   uyyval.pair = install_pair0(uyyvsp[-2].string, uyyvsp[-1].op, uyyvsp[0].string);   
+                   uyyval.pair = grad_create_pair0(uyyvsp[-2].string, uyyvsp[-1].op, uyyvsp[0].string);   
            ;
     break;}
 case 15:
-#line 143 "users.y"
+#line 146 "users.y"
 {
                    YYERROR;
            ;
     break;}
 case 16:
-#line 149 "users.y"
+#line 152 "users.y"
 {
-                   uyyval.op = OPERATOR_EQUAL;
+                   uyyval.op = grad_operator_equal;
            ;
     break;}
 case 17:
-#line 153 "users.y"
+#line 156 "users.y"
 {
-                   uyyval.op = OPERATOR_LESS_THAN;
+                   uyyval.op = grad_operator_less_than;
            ;
     break;}
 case 18:
-#line 157 "users.y"
+#line 160 "users.y"
 { 
-                   uyyval.op = OPERATOR_GREATER_THAN;
+                   uyyval.op = grad_operator_greater_than;
            ;
     break;}
 case 19:
-#line 161 "users.y"
+#line 164 "users.y"
 {
-                   uyyval.op = OPERATOR_NOT_EQUAL;
+                   uyyval.op = grad_operator_not_equal;
            ;
     break;}
 case 20:
-#line 165 "users.y"
+#line 168 "users.y"
 {
-                   uyyval.op = OPERATOR_LESS_EQUAL;
+                   uyyval.op = grad_operator_less_equal;
            ;
     break;}
 case 21:
-#line 169 "users.y"
+#line 172 "users.y"
 {
-                   uyyval.op = OPERATOR_GREATER_EQUAL;
+                   uyyval.op = grad_operator_greater_equal;
            ;
     break;}
 }
@@ -1081,41 +1084,41 @@ uyyerrhandle:
     }
   return 1;
 }
-#line 178 "users.y"
+#line 181 "users.y"
 
 
 int
 uyyerror(char *s)
 {
-        radlog(L_ERR, "%s:%d: %s", source_filename, source_line_num, s);
+	grad_log_loc(L_ERR, &grad_parser_source_locus, "%s", s);
 	return 0;
 }
 
-VALUE_PAIR *
-install_pair0(char *name, int op, char *valstr)
+grad_avp_t *
+grad_create_pair0(char *name, int op, char *valstr)
 {
-	return install_pair(source_filename, source_line_num, 
-			    name, op, valstr);
+	return grad_create_pair(&grad_parser_source_locus, name, op, valstr);
 }
 
-VALUE_PAIR *
-install_pair(char *file, int line, char *name, int op, char *valstr)
+grad_avp_t *
+grad_create_pair(grad_locus_t *loc, char *name, 
+                 enum grad_operator op, char *valstr)
 {
-        DICT_ATTR       *attr = NULL;
-        DICT_VALUE      *dval;
-        VALUE_PAIR      *pair, *pair2;
+        grad_dict_attr_t *attr = NULL;
+        grad_dict_value_t *dval;
+        grad_avp_t *pair, *pair2;
         char *s;
         int x;
         time_t timeval;
         struct tm *tm, tms;
         
-        if ((attr = attr_name_to_dict(name)) == (DICT_ATTR *)NULL) {
-                radlog(L_ERR, _("%s:%d: unknown attribute `%s'"),
-                       file, line, name);
+        if ((attr = grad_attr_name_to_dict(name)) == NULL) {
+                grad_log_loc(L_ERR, loc, _("unknown attribute `%s'"),
+			     name);
                 return NULL;
         }
 
-        pair = avp_alloc();
+        pair = grad_avp_alloc();
         
         pair->next = NULL;
         pair->name = attr->name;
@@ -1125,37 +1128,37 @@ install_pair(char *file, int line, char *name, int op, char *valstr)
         pair->operator = op;
 
         if (valstr[0] == '=') {
-                pair->eval = 1;
-                pair->avp_strvalue = estrdup(valstr+1);
+                pair->eval_type = eval_interpret;
+                pair->avp_strvalue = grad_estrdup(valstr+1);
                 pair->avp_strlength = strlen(pair->avp_strvalue);
                 return pair;
         }
 
-        pair->eval = 0;
+        pair->eval_type = eval_const;
         
         switch (pair->type) {
         case TYPE_STRING:
                 if (pair->attribute == DA_EXEC_PROGRAM ||
                     pair->attribute == DA_EXEC_PROGRAM_WAIT) {
                         if (valstr[0] != '/' && valstr[0] != '|') {
-                                radlog(L_ERR,
-				     _("%s:%d: %s: not an absolute pathname"),
-                                       file, line, name);
-                                avp_free(pair);
+                                grad_log_loc(L_ERR, loc,
+					     _("%s: not an absolute pathname"),
+					     name);
+                                grad_avp_free(pair);
                                 return NULL;
                         }
                 }
 
-		pair->avp_strvalue = estrdup(valstr);
+		pair->avp_strvalue = grad_estrdup(valstr);
                 pair->avp_strlength = strlen(pair->avp_strvalue);
 		
 		if (attr->parser && attr->parser(pair, &s)) {
-			radlog(L_ERR, "%s:%d: %s %s: %s",
-			       file, line,
-			       _("attribute"),
-			       pair->name, s);
+			grad_log_loc(L_ERR, loc,
+				     "%s %s: %s",
+				     _("attribute"),
+				     pair->name, s);
 			free(s);
-			avp_free(pair);
+			grad_avp_free(pair);
 			return NULL;
 		}
 
@@ -1172,18 +1175,20 @@ install_pair(char *file, int line, char *name, int op, char *valstr)
                                         break;
                         if (*s) {
                                 pair->type = TYPE_STRING;
-                                pair->avp_strvalue = estrdup(valstr);
+                                pair->avp_strvalue = grad_estrdup(valstr);
                                 pair->avp_strlength = strlen(pair->avp_strvalue);
                                 break;
                         }
                 }
                 if (isdigit(*valstr)) {
                         pair->avp_lvalue = atoi(valstr);
-                } else if ((dval = value_name_to_value(valstr, pair->attribute)) == NULL) {
-                        avp_free(pair);
-                        radlog(L_ERR,
-		       _("%s:%d: value %s is not declared for attribute %s"),
-			       file, line, valstr, name);
+                } else if ((dval = grad_value_name_to_value(valstr,
+							    pair->attribute))
+			         == NULL) {
+                        grad_avp_free(pair);
+                        grad_log_loc(L_ERR, loc,
+				_("value %s is not declared for attribute %s"),
+				     valstr, name);
                         return NULL;
                 } else {
                         pair->avp_lvalue = dval->value;
@@ -1192,7 +1197,7 @@ install_pair(char *file, int line, char *name, int op, char *valstr)
 
         case TYPE_IPADDR:
                 if (pair->attribute != DA_FRAMED_IP_ADDRESS) {
-                        pair->avp_lvalue = ip_gethostaddr(valstr);
+                        pair->avp_lvalue = grad_ip_gethostaddr(valstr);
                 } else {
                         /*
                          *      We allow a "+" at the end to
@@ -1208,43 +1213,41 @@ install_pair(char *file, int line, char *name, int op, char *valstr)
                                         x = 1;
                                 }
                         }
-                        pair->avp_lvalue = ip_gethostaddr(valstr);
+                        pair->avp_lvalue = grad_ip_gethostaddr(valstr);
 
-                        /*
-                         *      Add an extra (hidden) attribute.
-                         */
-                        pair2 = avp_alloc();
-                        
-                        pair2->name = "Add-Port-To-IP-Address";
-                        pair2->attribute = DA_ADD_PORT_TO_IP_ADDRESS;
-                        pair2->type = TYPE_INTEGER;
-                        pair2->avp_lvalue = x;
-                        pair2->next = pair;
-                        pair = pair2;
+			if (x) {
+				char *s;
+				asprintf(&s, "%lu+%%{NAS-Port-Id}",
+					 pair->avp_lvalue);
+				pair->avp_strvalue = grad_estrdup(s);
+				pair->avp_strlength = strlen(pair->avp_strvalue);
+				pair->eval_type = eval_interpret;
+				free(s);
+			}
                 }
                 break;
                 
         case TYPE_DATE:
                 timeval = time(0);
                 tm = localtime_r(&timeval, &tms);
-                if (user_gettime(valstr, tm)) {
-                        radlog(L_ERR,
-                                _("%s:%d: %s: can't parse date"),
-                               file, line, name);
-                        avp_free(pair);
+                if (grad_parse_time_string(valstr, tm)) {
+                        grad_log_loc(L_ERR, loc, _("%s: can't parse date"),
+				     name);
+                        grad_avp_free(pair);
                         return NULL;
                 }
 #ifdef TIMELOCAL
-                pair->avp_lvalue = (UINT4)timelocal(tm);
+                pair->avp_lvalue = (grad_uint32_t)timelocal(tm);
 #else /* TIMELOCAL */
-                pair->avp_lvalue = (UINT4)mktime(tm);
+                pair->avp_lvalue = (grad_uint32_t)mktime(tm);
 #endif /* TIMELOCAL */
                 break;
 
         default:
-                radlog(L_ERR, _("%s:%d: %s: unknown attribute type %d"),
-		       file, line, name, pair->type);
-                avp_free(pair);
+                grad_log_loc(L_ERR, loc,
+			     _("%s: unknown attribute type %d"),
+			     name, pair->type);
+                grad_avp_free(pair);
                 return NULL;
         }
 
@@ -1254,29 +1257,26 @@ install_pair(char *file, int line, char *name, int op, char *valstr)
 extern int uyydebug;
 
 int
-parse_file(char *file, void *c, int (*f)())
+grad_parse_rule_file(char *file, void *c, register_rule_fp f)
 {
         int rc;
         
-        if (init_lex(file))
+        if (grad_parser_lex_init(file))
                 return -1;
         closure = c;
         add_entry = f;
 
         uyydebug = 0;
         rc = uyyparse();
-        done_lex();
+        grad_parser_lex_finish();
         return rc;
 }
 
 void
-enable_usr_dbg(int val)
+grad_enable_rule_debug(int val)
 {
         uyydebug = val;
-        if (uyydebug)
-                radlog(L_NOTICE, _("%s:%d: enabled userfile parser debugging"),
-                       source_filename, source_line_num);
-        else
-                radlog(L_NOTICE, _("%s:%d: disabled userfile parser debugging"),
-                       source_filename, source_line_num);
+	grad_log_loc(L_NOTICE, &grad_parser_source_locus,
+		     uyydebug ? _("enabled userfile parser debugging") :
+             	 	       _("disabled userfile parser debugging"));
 }
