@@ -1,6 +1,6 @@
 /* 
    This file is part of GNU Radius SNMP Library.
-   Copyright (C) 2001,2003 Free Software Foundation, Inc.
+   Copyright (C) 2001,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -22,8 +22,8 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
-#include <asn1.h>
-#include <snmp.h>
+#include <snmp/asn1.h>
+#include <snmp/snmp.h>
 
 #ifndef SNMP_SET_ERRNO
 # define SNMP_SET_ERRNO(e)
